@@ -16,7 +16,6 @@
 from __future__ import print_function
 
 import argparse
-import base64
 import contextlib
 import gzip
 import json
@@ -26,6 +25,7 @@ import subprocess
 import sys
 import tempfile
 
+from oslo_serialization import base64
 from oslo_utils import strutils
 
 from ironicclient.common.i18n import _
@@ -34,9 +34,7 @@ from ironicclient import exc
 
 class HelpFormatter(argparse.HelpFormatter):
     def start_section(self, heading):
-        # Title-case the headings
-        heading = '%s%s' % (heading[0].upper(), heading[1:])
-        super(HelpFormatter, self).start_section(heading)
+        super(HelpFormatter, self).start_section(heading.capitalize())
 
 
 def define_command(subparsers, command, callback, cmd_mapper):
@@ -57,8 +55,13 @@ def define_command(subparsers, command, callback, cmd_mapper):
     subparser.add_argument('-h', '--help', action='help',
                            help=argparse.SUPPRESS)
     cmd_mapper[command] = subparser
+    required_args = subparser.add_argument_group(_("Required arguments"))
+
     for (args, kwargs) in arguments:
-        subparser.add_argument(*args, **kwargs)
+        if kwargs.get('required'):
+            required_args.add_argument(*args, **kwargs)
+        else:
+            subparser.add_argument(*args, **kwargs)
     subparser.set_defaults(func=callback)
 
 
@@ -267,7 +270,7 @@ def make_configdrive(path):
             g.close()
 
             tmpzipfile.seek(0)
-            return base64.b64encode(tmpzipfile.read())
+            return base64.encode_as_bytes(tmpzipfile.read())
 
 
 def check_empty_arg(arg, arg_descriptor):
