@@ -69,6 +69,55 @@ class ProvisionStateBaremetalNode(command.Command):
             cleansteps=clean_steps)
 
 
+class ProvisionStateWithWait(ProvisionStateBaremetalNode):
+    """Provision state class adding --wait flag."""
+
+    log = logging.getLogger(__name__ + ".ProvisionStateWithWait")
+
+    def get_parser(self, prog_name):
+        parser = super(ProvisionStateWithWait, self).get_parser(prog_name)
+
+        desired_state = v1_utils.PROVISION_ACTIONS.get(
+            self.PROVISION_STATE)['expected_state']
+        parser.add_argument(
+            '--wait',
+            type=int,
+            dest='wait_timeout',
+            default=None,
+            metavar='<time-out>',
+            const=0,
+            nargs='?',
+            help=("Wait for a node to reach the desired state, %(state)s. "
+                  "Optionally takes a timeout value (in seconds). The "
+                  "default value is 0, meaning it will wait indefinitely.") %
+            {'state': desired_state})
+        return parser
+
+    def take_action(self, parsed_args):
+        super(ProvisionStateWithWait, self).take_action(parsed_args)
+        self.log.debug("take_action(%s)", parsed_args)
+
+        if (parsed_args.wait_timeout is None):
+            return
+
+        baremetal_client = self.app.client_manager.baremetal
+        wait_args = v1_utils.PROVISION_ACTIONS.get(
+            parsed_args.provision_state)
+        if wait_args is None:
+            # This should never happen in reality, but checking just in case
+            raise exc.CommandError(
+                _("'--wait is not supported for provision state '%s'")
+                % parsed_args.provision_state)
+
+        print(_('Waiting for provision state %(state)s on node %(node)s') %
+              {'state': wait_args['expected_state'], 'node': parsed_args.node})
+
+        baremetal_client.node.wait_for_provision_state(
+            parsed_args.node,
+            timeout=parsed_args.wait_timeout,
+            **wait_args)
+
+
 class AbortBaremetalNode(ProvisionStateBaremetalNode):
     """Set provision state of baremetal node to 'abort'"""
 
@@ -76,7 +125,7 @@ class AbortBaremetalNode(ProvisionStateBaremetalNode):
     PROVISION_STATE = 'abort'
 
 
-class AdoptBaremetalNode(ProvisionStateBaremetalNode):
+class AdoptBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'adopt'"""
 
     log = logging.getLogger(__name__ + ".AdoptBaremetalNode")
@@ -157,7 +206,7 @@ class BootdeviceShowBaremetalNode(command.ShowOne):
         return zip(*sorted(info.items()))
 
 
-class CleanBaremetalNode(ProvisionStateBaremetalNode):
+class CleanBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'clean'"""
 
     log = logging.getLogger(__name__ + ".CleanBaremetalNode")
@@ -374,7 +423,7 @@ class DeleteBaremetal(DeleteBaremetalNode):
         super(DeleteBaremetal, self).take_action(parsed_args)
 
 
-class DeployBaremetalNode(ProvisionStateBaremetalNode):
+class DeployBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'deploy'"""
 
     log = logging.getLogger(__name__ + ".DeployBaremetalNode")
@@ -394,7 +443,7 @@ class DeployBaremetalNode(ProvisionStateBaremetalNode):
         return parser
 
 
-class InspectBaremetalNode(ProvisionStateBaremetalNode):
+class InspectBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'inspect'"""
 
     log = logging.getLogger(__name__ + ".InspectBaremetalNode")
@@ -433,19 +482,31 @@ class ListBaremetalNode(command.Lister):
                  '(asc or desc) (default: asc). Multiple fields and '
                  'directions can be specified, separated by comma.',
         )
-        parser.add_argument(
+        maint_group = parser.add_mutually_exclusive_group(required=False)
+        maint_group.add_argument(
             '--maintenance',
             dest='maintenance',
             action='store_true',
             default=False,
-            help="List nodes in maintenance mode.",
+            help="Limit list to nodes in maintenance mode",
         )
-        parser.add_argument(
-            '--associated',
-            dest='associated',
+        maint_group.add_argument(
+            '--no-maintenance',
+            dest='no_maintenance',
             action='store_true',
             default=False,
-            help="List only nodes associated with an instance."
+            help="Limit list to nodes not in maintenance mode",
+        )
+        associated_group = parser.add_mutually_exclusive_group()
+        associated_group.add_argument(
+            '--associated',
+            action='store_true',
+            help=_('List only nodes associated with an instance.'),
+        )
+        associated_group.add_argument(
+            '--unassociated',
+            action='store_true',
+            help=_('List only nodes not associated with an instance.'),
         )
         parser.add_argument(
             '--provision-state',
@@ -498,9 +559,13 @@ class ListBaremetalNode(command.Lister):
         params['limit'] = parsed_args.limit
         params['marker'] = parsed_args.marker
         if parsed_args.associated:
-            params['associated'] = parsed_args.associated
+            params['associated'] = True
+        if parsed_args.unassociated:
+            params['associated'] = False
         if parsed_args.maintenance:
-            params['maintenance'] = parsed_args.maintenance
+            params['maintenance'] = True
+        elif parsed_args.no_maintenance:
+            params['maintenance'] = False
         if parsed_args.provision_state:
             params['provision_state'] = parsed_args.provision_state
         if parsed_args.resource_class:
@@ -599,7 +664,7 @@ class MaintenanceUnsetBaremetalNode(command.Command):
             False)
 
 
-class ManageBaremetalNode(ProvisionStateBaremetalNode):
+class ManageBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'manage'"""
 
     log = logging.getLogger(__name__ + ".ManageBaremetalNode")
@@ -741,7 +806,7 @@ class PowerBaremetalNode(command.Command):
             timeout=parsed_args.power_timeout)
 
 
-class ProvideBaremetalNode(ProvisionStateBaremetalNode):
+class ProvideBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'provide'"""
 
     log = logging.getLogger(__name__ + ".ProvideBaremetalNode")
@@ -789,7 +854,7 @@ class RebootBaremetalNode(command.Command):
             timeout=parsed_args.power_timeout)
 
 
-class RebuildBaremetalNode(ProvisionStateBaremetalNode):
+class RebuildBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'rebuild'"""
 
     log = logging.getLogger(__name__ + ".RebuildBaremetalNode")
@@ -1015,7 +1080,7 @@ class ShowBaremetal(ShowBaremetalNode):
         return super(ShowBaremetal, self).take_action(parsed_args)
 
 
-class UndeployBaremetalNode(ProvisionStateBaremetalNode):
+class UndeployBaremetalNode(ProvisionStateWithWait):
     """Set provision state of baremetal node to 'deleted'"""
 
     log = logging.getLogger(__name__ + ".UndeployBaremetalNode")
@@ -1273,3 +1338,27 @@ class VifDetachBaremetalNode(command.Command):
 
         baremetal_client = self.app.client_manager.baremetal
         baremetal_client.node.vif_detach(parsed_args.node, parsed_args.vif_id)
+
+
+class InjectNmiBaremetalNode(command.Command):
+    """Inject NMI to baremetal node"""
+
+    log = logging.getLogger(__name__ + ".InjectNmiBaremetalNode")
+
+    def get_parser(self, prog_name):
+        parser = super(InjectNmiBaremetalNode, self).get_parser(prog_name)
+
+        parser.add_argument(
+            'node',
+            metavar='<node>',
+            help=_("Name or UUID of the node.")
+        )
+
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)", parsed_args)
+
+        baremetal_client = self.app.client_manager.baremetal
+
+        baremetal_client.node.inject_nmi(parsed_args.node)
