@@ -120,6 +120,33 @@ class TestCreateBaremetalPort(TestBaremetalPort):
                                        'switch_id=aa:bb:cc:dd:ee:ff'])]
         )
 
+    def test_baremetal_port_create_portgroup_uuid(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_address,
+            '--node', baremetal_fakes.baremetal_uuid,
+            '--port-group', baremetal_fakes.baremetal_portgroup_uuid,
+        ]
+
+        verifylist = [
+            ('node_uuid', baremetal_fakes.baremetal_uuid),
+            ('address', baremetal_fakes.baremetal_port_address),
+            ('portgroup_uuid', baremetal_fakes.baremetal_portgroup_uuid)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        args = {
+            'address': baremetal_fakes.baremetal_port_address,
+            'node_uuid': baremetal_fakes.baremetal_uuid,
+            'portgroup_uuid': baremetal_fakes.baremetal_portgroup_uuid
+        }
+
+        self.baremetal_mock.port.create.assert_called_once_with(**args)
+
 
 class TestShowBaremetalPort(TestBaremetalPort):
     def setUp(self):
@@ -204,6 +231,14 @@ class TestBaremetalPortUnset(TestBaremetalPort):
                           self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_baremetal_port_unset_no_property(self):
+        arglist = [baremetal_fakes.baremetal_port_uuid]
+        verifylist = [('port', baremetal_fakes.baremetal_port_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.assertFalse(self.baremetal_mock.port.update.called)
+
     def test_baremetal_port_unset_extra(self):
         arglist = ['port', '--extra', 'foo']
         verifylist = [('port', 'port'),
@@ -231,6 +266,18 @@ class TestBaremetalPortUnset(TestBaremetalPort):
             [{'path': '/extra/foo', 'op': 'remove'},
              {'path': '/extra/bar', 'op': 'remove'}])
 
+    def test_baremetal_port_unset_portgroup_uuid(self):
+        arglist = ['port', '--port-group']
+        verifylist = [('port', 'port'),
+                      ('portgroup', True)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            'port',
+            [{'path': '/portgroup_uuid', 'op': 'remove'}])
+
 
 class TestBaremetalPortSet(TestBaremetalPort):
     def setUp(self):
@@ -289,6 +336,23 @@ class TestBaremetalPortSet(TestBaremetalPort):
             'port',
             [{'path': '/extra/foo', 'value': 'bar', 'op': 'add'}])
 
+    def test_baremetal_port_set_portgroup_uuid(self):
+        new_portgroup_uuid = '1111-111111-1111'
+        arglist = [
+            baremetal_fakes.baremetal_port_uuid,
+            '--port-group', new_portgroup_uuid]
+        verifylist = [
+            ('port', baremetal_fakes.baremetal_port_uuid),
+            ('portgroup_uuid', new_portgroup_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            baremetal_fakes.baremetal_port_uuid,
+            [{'path': '/portgroup_uuid', 'value': new_portgroup_uuid,
+              'op': 'add'}])
+
     def test_baremetal_port_set_no_options(self):
         arglist = []
         verifylist = []
@@ -296,6 +360,14 @@ class TestBaremetalPortSet(TestBaremetalPort):
                           self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_baremetal_port_set_no_property(self):
+        arglist = [baremetal_fakes.baremetal_port_uuid]
+        verifylist = [('port', baremetal_fakes.baremetal_port_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.assertFalse(self.baremetal_mock.port.update.called)
+
 
 class TestBaremetalPortDelete(TestBaremetalPort):
     def setUp(self):
@@ -419,6 +491,21 @@ class TestBaremetalPortList(TestBaremetalPort):
         }
         self.baremetal_mock.port.list.assert_called_with(**kwargs)
 
+    def test_baremetal_port_list_portgroup(self):
+        arglist = ['--port-group', baremetal_fakes.baremetal_portgroup_uuid]
+        verifylist = [('portgroup', baremetal_fakes.baremetal_portgroup_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        # DisplayCommandBase.take_action() returns two tuples
+        columns, data = self.cmd.take_action(parsed_args)
+
+        kwargs = {
+            'portgroup': baremetal_fakes.baremetal_portgroup_uuid,
+            'marker': None,
+            'limit': None,
+        }
+        self.baremetal_mock.port.list.assert_called_with(**kwargs)
+
     def test_baremetal_port_list_long(self):
         arglist = ['--long']
         verifylist = [('detail', True)]
@@ -434,8 +521,8 @@ class TestBaremetalPortList(TestBaremetalPort):
         self.baremetal_mock.port.list.assert_called_with(**kwargs)
 
         collist = ('UUID', 'Address', 'Created At', 'Extra', 'Node UUID',
-                   'Local Link Connection', 'PXE boot enabled', 'Updated At',
-                   'Internal Info')
+                   'Local Link Connection', 'Portgroup UUID',
+                   'PXE boot enabled', 'Updated At', 'Internal Info')
         self.assertEqual(collist, columns)
 
         datalist = ((
@@ -447,6 +534,7 @@ class TestBaremetalPortList(TestBaremetalPort):
             '',
             '',
             '',
+            '',
             ''
         ), )
         self.assertEqual(datalist, tuple(data))
