@@ -423,6 +423,10 @@ class NodeManager(base.CreateManager):
         path = "%s/management/boot_device" % node_uuid
         return self._get_as_dict(path)
 
+    def inject_nmi(self, node_uuid):
+        path = "%s/management/inject_nmi" % node_uuid
+        return self.update(path, {}, http_method='PUT')
+
     def get_supported_boot_devices(self, node_uuid):
         path = "%s/management/boot_device/supported" % node_uuid
         return self._get_as_dict(path)
