@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -15,15 +13,19 @@
 
 from ironicclient.common import utils
 from ironicclient.v1 import chassis_shell
+from ironicclient.v1 import create_resources_shell
 from ironicclient.v1 import driver_shell
 from ironicclient.v1 import node_shell
 from ironicclient.v1 import port_shell
+from ironicclient.v1 import portgroup_shell
 
 COMMAND_MODULES = [
     chassis_shell,
     node_shell,
     port_shell,
+    portgroup_shell,
     driver_shell,
+    create_resources_shell,
 ]
 
 
@@ -33,8 +35,9 @@ def enhance_parser(parser, subparsers, cmd_mapper):
     Take a basic (nonversioned) parser and enhance it with
     commands and options specific for this version of API.
 
-    :param parser: top level parser :param subparsers: top level
-        parser's subparsers collection where subcommands will go
+    :param parser: top level parser
+    :param subparsers: top level parser's subparsers collection
+                       where subcommands will go
     """
     for command_module in COMMAND_MODULES:
         utils.define_commands_from_module(subparsers, command_module,
